@@ -12,7 +12,9 @@ import hashlib
 import json
 import logging
 import os
+import re
 import shutil
+import tarfile
 import tempfile
 
 from concurrent.futures import ThreadPoolExecutor
@@ -25,63 +27,26 @@ from sos.cleaner.parsers.mac_parser import SoSMacParser
 from sos.cleaner.parsers.hostname_parser import SoSHostnameParser
 from sos.cleaner.parsers.keyword_parser import SoSKeywordParser
 from sos.cleaner.parsers.username_parser import SoSUsernameParser
-from sos.cleaner.archives.sos import (SoSReportArchive, SoSReportDirectory,
-                                      SoSCollectorArchive,
-                                      SoSCollectorDirectory)
-from sos.cleaner.archives.generic import DataDirArchive, TarballArchive
-from sos.cleaner.archives.insights import InsightsArchive
+from sos.cleaner.obfuscation_archive import SoSObfuscationArchive
 from sos.utilities import get_human_readable
 from textwrap import fill
 
 
 class SoSCleaner(SoSComponent):
-    """
-    This function is designed to obfuscate potentially sensitive information
-    from an sos report archive in a consistent and reproducible manner.
-
-    It may either be invoked during the creation of a report by using the
-    --clean option in the report command, or may be used on an already existing
-    archive by way of 'sos clean'.
-
-    The target of obfuscation are items such as IP addresses, MAC addresses,
-    hostnames, usernames, and also keywords provided by users via the
-    --keywords and/or --keyword-file options.
-
-    For every collection made in a report the collection is parsed for such
-    items, and when items are found SoS will generate an obfuscated replacement
-    for it, and in all places that item is found replace the text with the
-    obfuscated replacement mapped to it. These mappings are saved locally so
-    that future iterations will maintain the same consistent obfuscation
-    pairing.
-
-    In the case of IP addresses, support is for IPv4 and efforts are made to
-    keep network topology intact so that later analysis is as accurate and
-    easily understandable as possible. If an IP address is encountered that we
-    cannot determine the netmask for, a random IP address is used instead.
-
-    For hostnames, domains are obfuscated as whole units, leaving the TLD in
-    place.
-
-    For instance, 'example.com' may be obfuscated to 'obfuscateddomain0.com'
-    and 'foo.example.com' may end up being 'obfuscateddomain1.com'.
-
-    Users will be notified of a 'mapping' file that records all items and the
-    obfuscated counterpart mapped to them for ease of reference later on. This
-    file should be kept private.
+    """Take an sos report, or collection of sos reports, and scrub them of
+    potentially sensitive data such as IP addresses, hostnames, MAC addresses,
+    etc.. that are not obfuscated by individual plugins
     """
 
     desc = "Obfuscate sensitive networking information in a report"
 
     arg_defaults = {
-        'archive_type': 'auto',
         'domains': [],
-        'disable_parsers': [],
         'jobs': 4,
         'keywords': [],
         'keyword_file': None,
         'map_file': '/etc/sos/cleaner/default_mapping',
         'no_update': False,
-        'keep_binary_files': False,
         'target': '',
         'usernames': []
     }
@@ -104,14 +69,13 @@ class SoSCleaner(SoSComponent):
             self.from_cmdline = False
             if not hasattr(self.opts, 'jobs'):
                 self.opts.jobs = 4
-            self.opts.archive_type = 'auto'
             self.soslog = logging.getLogger('sos')
             self.ui_log = logging.getLogger('sos_ui')
             # create the tmp subdir here to avoid a potential race condition
             # when obfuscating a SoSCollector run during archive extraction
             os.makedirs(os.path.join(self.tmpdir, 'cleaner'), exist_ok=True)
 
-        self.cleaner_mapping = self.load_map_file()
+        self.validate_map_file()
         os.umask(0o77)
         self.in_place = in_place
         self.hash_name = self.policy.get_preferred_hash_name()
@@ -119,38 +83,14 @@ class SoSCleaner(SoSComponent):
         self.cleaner_md = self.manifest.components.add_section('cleaner')
 
         self.parsers = [
-            SoSHostnameParser(self.cleaner_mapping, self.opts.domains),
-            SoSIPParser(self.cleaner_mapping),
-            SoSMacParser(self.cleaner_mapping),
-            SoSKeywordParser(self.cleaner_mapping, self.opts.keywords,
+            SoSHostnameParser(self.opts.map_file, self.opts.domains),
+            SoSIPParser(self.opts.map_file),
+            SoSMacParser(self.opts.map_file),
+            SoSKeywordParser(self.opts.map_file, self.opts.keywords,
                              self.opts.keyword_file),
-            SoSUsernameParser(self.cleaner_mapping, self.opts.usernames)
+            SoSUsernameParser(self.opts.map_file, self.opts.usernames)
         ]
 
-        for _parser in self.opts.disable_parsers:
-            for _loaded in self.parsers:
-                _loaded_name = _loaded.name.lower().split('parser')[0].strip()
-                if _parser.lower().strip() == _loaded_name:
-                    self.log_info("Disabling parser: %s" % _loaded_name)
-                    self.ui_log.warn(
-                        "Disabling the '%s' parser. Be aware that this may "
-                        "leave sensitive plain-text data in the archive."
-                        % _parser
-                    )
-                    self.parsers.remove(_loaded)
-
-        self.archive_types = [
-            SoSReportDirectory,
-            SoSReportArchive,
-            SoSCollectorDirectory,
-            SoSCollectorArchive,
-            InsightsArchive,
-            # make sure these two are always last as they are fallbacks
-            DataDirArchive,
-            TarballArchive
-        ]
-        self.nested_archive = None
-
         self.log_info("Cleaner initialized. From cmdline: %s"
                       % self.from_cmdline)
 
@@ -173,18 +113,12 @@ class SoSCleaner(SoSComponent):
             _fmt = _fmt + fill(line, width, replace_whitespace=False) + '\n'
         return _fmt
 
-    @classmethod
-    def display_help(cls, section):
-        section.set_title("SoS Cleaner Detailed Help")
-        section.add_text(cls.__doc__)
-
-    def load_map_file(self):
+    def validate_map_file(self):
         """Verifies that the map file exists and has usable content.
 
         If the provided map file does not exist, or it is empty, we will print
         a warning and continue on with cleaning building a fresh map
         """
-        _conf = {}
         default_map = '/etc/sos/cleaner/default_mapping'
         if os.path.isdir(self.opts.map_file):
             raise Exception("Requested map file %s is a directory"
@@ -194,17 +128,6 @@ class SoSCleaner(SoSComponent):
                 self.log_error(
                     "ERROR: map file %s does not exist, will not load any "
                     "obfuscation matches" % self.opts.map_file)
-        else:
-            with open(self.opts.map_file, 'r') as mf:
-                try:
-                    _conf = json.load(mf)
-                except json.JSONDecodeError:
-                    self.log_error("ERROR: Unable to parse map file, json is "
-                                   "malformed. Will not load any mappings.")
-                except Exception as err:
-                    self.log_error("ERROR: Could not load '%s': %s"
-                                   % (self.opts.map_file, err))
-        return _conf
 
     def print_disclaimer(self):
         """When we are directly running `sos clean`, rather than hooking into
@@ -232,8 +155,6 @@ third party.
             except KeyboardInterrupt:
                 self.ui_log.info("\nExiting on user cancel")
                 self._exit(130)
-            except Exception as e:
-                self._exit(1, e)
 
     @classmethod
     def add_parser_options(cls, parser):
@@ -244,17 +165,8 @@ third party.
         )
         clean_grp.add_argument('target', metavar='TARGET',
                                help='The directory or archive to obfuscate')
-        clean_grp.add_argument('--archive-type', default='auto',
-                               choices=['auto', 'report', 'collect',
-                                        'insights', 'data-dir', 'tarball'],
-                               help=('Specify what kind of archive the target '
-                                     'was generated as'))
         clean_grp.add_argument('--domains', action='extend', default=[],
                                help='List of domain names to obfuscate')
-        clean_grp.add_argument('--disable-parsers', action='extend',
-                               default=[], dest='disable_parsers',
-                               help=('Disable specific parsers, so that those '
-                                     'elements are not obfuscated'))
         clean_grp.add_argument('-j', '--jobs', default=4, type=int,
                                help='Number of concurrent archives to clean')
         clean_grp.add_argument('--keywords', action='extend', default=[],
@@ -269,13 +181,8 @@ third party.
                                      'file for obfuscation'))
         clean_grp.add_argument('--no-update', dest='no_update', default=False,
                                action='store_true',
-                               help='Do not update the --map-file with new '
+                               help='Do not update the --map file with new '
                                     'mappings from this run')
-        clean_grp.add_argument('--keep-binary-files', default=False,
-                               action='store_true',
-                               dest='keep_binary_files',
-                               help='Keep unprocessable binary files in the '
-                                    'archive instead of removing them')
         clean_grp.add_argument('--usernames', dest='usernames', default=[],
                                action='extend',
                                help='List of usernames to obfuscate')
@@ -293,28 +200,58 @@ third party.
 
         In the event the target path is not an archive, abort.
         """
-        _arc = None
-        if self.opts.archive_type != 'auto':
-            check_type = self.opts.archive_type.replace('-', '_')
-            for archive in self.archive_types:
-                if archive.type_name == check_type:
-                    _arc = archive(self.opts.target, self.tmpdir)
+        if not tarfile.is_tarfile(self.opts.target):
+            self.ui_log.error(
+                "Invalid target: must be directory or tar archive"
+            )
+            self._exit(1)
+
+        archive = tarfile.open(self.opts.target)
+        self.arc_name = self.opts.target.split('/')[-1].split('.')[:-2][0]
+
+        try:
+            archive.getmember(os.path.join(self.arc_name, 'sos_logs'))
+        except Exception:
+            # this is not an sos archive
+            self.ui_log.error("Invalid target: not an sos archive")
+            self._exit(1)
+
+        # see if there are archives within this archive
+        nested_archives = []
+        for _file in archive.getmembers():
+            if (re.match('sosreport-.*.tar', _file.name.split('/')[-1]) and not
+               (_file.name.endswith('.md5') or
+               _file.name.endswith('.sha256'))):
+                nested_archives.append(_file.name.split('/')[-1])
+
+        if nested_archives:
+            self.log_info("Found nested archive(s), extracting top level")
+            nested_path = self.extract_archive(archive)
+            for arc_file in os.listdir(nested_path):
+                if re.match('sosreport.*.tar.*', arc_file):
+                    self.report_paths.append(os.path.join(nested_path,
+                                                          arc_file))
+            # add the toplevel extracted archive
+            self.report_paths.append(nested_path)
         else:
-            for arc in self.archive_types:
-                if arc.check_is_type(self.opts.target):
-                    _arc = arc(self.opts.target, self.tmpdir)
-                    break
-        if not _arc:
-            return
-        self.report_paths.append(_arc)
-        if _arc.is_nested:
-            self.report_paths.extend(_arc.get_nested_archives())
-            # We need to preserve the top level archive until all
-            # nested archives are processed
-            self.report_paths.remove(_arc)
-            self.nested_archive = _arc
-        if self.nested_archive:
-            self.nested_archive.ui_name = self.nested_archive.description
+            self.report_paths.append(self.opts.target)
+
+        archive.close()
+
+    def extract_archive(self, archive):
+        """Extract an archive into our tmpdir so that we may inspect it or
+        iterate through its contents for obfuscation
+
+        Positional arguments:
+
+            :param archive:     An open TarFile object for the archive
+
+        """
+        if not isinstance(archive, tarfile.TarFile):
+            archive = tarfile.open(archive)
+        path = os.path.join(self.tmpdir, 'cleaner')
+        archive.extractall(path)
+        return os.path.join(path, archive.name.split('/')[-1].split('.tar')[0])
 
     def execute(self):
         """SoSCleaner will begin by inspecting the TARGET option to determine
@@ -327,7 +264,6 @@ third party.
         be unpacked, cleaned, and repacked and the final top-level archive will
         then be repacked as well.
         """
-        self.arc_name = self.opts.target.split('/')[-1].split('.tar')[0]
         if self.from_cmdline:
             self.print_disclaimer()
         self.report_paths = []
@@ -335,17 +271,27 @@ third party.
             self.ui_log.error("Invalid target: no such file or directory %s"
                               % self.opts.target)
             self._exit(1)
-
-        self.inspect_target_archive()
+        if os.path.isdir(self.opts.target):
+            self.arc_name = self.opts.target.split('/')[-1]
+            for _file in os.listdir(self.opts.target):
+                if _file == 'sos_logs':
+                    self.report_paths.append(self.opts.target)
+                if (_file.startswith('sosreport') and
+                   (_file.endswith(".tar.gz") or _file.endswith(".tar.xz"))):
+                    self.report_paths.append(os.path.join(self.opts.target,
+                                                          _file))
+            if not self.report_paths:
+                self.ui_log.error("Invalid target: not an sos directory")
+                self._exit(1)
+        else:
+            self.inspect_target_archive()
 
         if not self.report_paths:
-            self.ui_log.error("No valid archives or directories found\n")
+            self.ui_log.error("No valid sos archives or directories found\n")
             self._exit(1)
 
         # we have at least one valid target to obfuscate
         self.completed_reports = []
-        self.preload_all_archives_into_maps()
-        self.generate_parser_item_regexes()
         self.obfuscate_report_paths()
 
         if not self.completed_reports:
@@ -368,7 +314,33 @@ third party.
 
         final_path = None
         if len(self.completed_reports) > 1:
-            arc_path = self.rebuild_nested_archive()
+            # we have an archive of archives, so repack the obfuscated tarball
+            arc_name = self.arc_name + '-obfuscated'
+            self.setup_archive(name=arc_name)
+            for arc in self.completed_reports:
+                if arc.is_tarfile:
+                    arc_dest = self.obfuscate_string(
+                        arc.final_archive_path.split('/')[-1]
+                    )
+                    self.archive.add_file(arc.final_archive_path,
+                                          dest=arc_dest)
+                    checksum = self.get_new_checksum(arc.final_archive_path)
+                    if checksum is not None:
+                        dname = self.obfuscate_string(
+                            "checksums/%s.%s" % (arc_dest, self.hash_name)
+                        )
+                        self.archive.add_string(checksum, dest=dname)
+                else:
+                    for dirname, dirs, files in os.walk(arc.archive_path):
+                        for filename in files:
+                            if filename.startswith('sosreport'):
+                                continue
+                            fname = os.path.join(dirname, filename)
+                            dnm = self.obfuscate_string(
+                                fname.split(arc.archive_name)[-1].lstrip('/')
+                            )
+                            self.archive.add_file(fname, dest=dnm)
+            arc_path = self.archive.finalize(self.opts.compression_type)
         else:
             arc = self.completed_reports[0]
             arc_path = arc.final_archive_path
@@ -379,7 +351,8 @@ third party.
                 )
                 with open(os.path.join(self.sys_tmp, chksum_name), 'w') as cf:
                     cf.write(checksum)
-            self.write_cleaner_log()
+
+        self.write_cleaner_log()
 
         final_path = self.obfuscate_string(
             os.path.join(self.sys_tmp, arc_path.split('/')[-1])
@@ -400,30 +373,6 @@ third party.
 
         self.cleanup()
 
-    def rebuild_nested_archive(self):
-        """Handles repacking the nested tarball, now containing only obfuscated
-        copies of the reports, log files, manifest, etc...
-        """
-        # we have an archive of archives, so repack the obfuscated tarball
-        arc_name = self.arc_name + '-obfuscated'
-        self.setup_archive(name=arc_name)
-        for archive in self.completed_reports:
-            arc_dest = archive.final_archive_path.split('/')[-1]
-            checksum = self.get_new_checksum(archive.final_archive_path)
-            if checksum is not None:
-                dname = "checksums/%s.%s" % (arc_dest, self.hash_name)
-                self.archive.add_string(checksum, dest=dname)
-        for dirn, dirs, files in os.walk(self.nested_archive.extracted_path):
-            for filename in files:
-                fname = os.path.join(dirn, filename)
-                dname = fname.split(self.nested_archive.extracted_path)[-1]
-                dname = dname.lstrip('/')
-                self.archive.add_file(fname, dest=dname)
-                # remove it now so we don't balloon our fs space needs
-                os.remove(fname)
-        self.write_cleaner_log(archive=True)
-        return self.archive.finalize(self.opts.compression_type)
-
     def compile_mapping_dict(self):
         """Build a dict that contains each parser's map as a key, with the
         contents as that key's value. This will then be written to disk in the
@@ -447,9 +396,8 @@ third party.
 
     def write_map_for_archive(self, _map):
         try:
-            map_path = os.path.join(
-                self.sys_tmp,
-                self.obfuscate_string("%s-private_map" % self.arc_name)
+            map_path = self.obfuscate_string(
+                os.path.join(self.sys_tmp, "%s-private_map" % self.arc_name)
             )
             return self.write_map_to_file(_map, map_path)
         except Exception as err:
@@ -473,7 +421,7 @@ third party.
                 self.log_error("Could not update mapping config file: %s"
                                % err)
 
-    def write_cleaner_log(self, archive=False):
+    def write_cleaner_log(self):
         """When invoked via the command line, the logging from SoSCleaner will
         not be added to the archive(s) it processes, so we need to write it
         separately to disk
@@ -486,10 +434,6 @@ third party.
             for line in self.sos_log_file.readlines():
                 logfile.write(line)
 
-        if archive:
-            self.obfuscate_file(log_name)
-            self.archive.add_file(log_name, dest="sos_logs/cleaner.log")
-
     def get_new_checksum(self, archive_path):
         """Calculate a new checksum for the obfuscated archive, as the previous
         checksum will no longer be valid
@@ -517,93 +461,19 @@ third party.
         be obfuscated concurrently.
         """
         try:
-            msg = (
-                "Found %s total reports to obfuscate, processing up to %s "
-                "concurrently\n" % (len(self.report_paths), self.opts.jobs)
-            )
-            self.ui_log.info(msg)
-            if self.opts.keep_binary_files:
-                self.ui_log.warning(
-                    "WARNING: binary files that potentially contain sensitive "
-                    "information will NOT be removed from the final archive\n"
-                )
+            if len(self.report_paths) > 1:
+                msg = ("Found %s total reports to obfuscate, processing up to "
+                       "%s concurrently\n"
+                       % (len(self.report_paths), self.opts.jobs))
+                self.ui_log.info(msg)
             pool = ThreadPoolExecutor(self.opts.jobs)
             pool.map(self.obfuscate_report, self.report_paths, chunksize=1)
             pool.shutdown(wait=True)
-            # finally, obfuscate the nested archive if one exists
-            if self.nested_archive:
-                self._replace_obfuscated_archives()
-                self.obfuscate_report(self.nested_archive)
         except KeyboardInterrupt:
             self.ui_log.info("Exiting on user cancel")
             os._exit(130)
 
-    def _replace_obfuscated_archives(self):
-        """When we have a nested archive, we need to rebuild the original
-        archive, which entails replacing the existing archives with their
-        obfuscated counterparts
-        """
-        for archive in self.completed_reports:
-            os.remove(archive.archive_path)
-            dest = self.nested_archive.extracted_path
-            tarball = archive.final_archive_path.split('/')[-1]
-            dest_name = os.path.join(dest, tarball)
-            shutil.move(archive.final_archive_path, dest)
-            archive.final_archive_path = dest_name
-
-    def generate_parser_item_regexes(self):
-        """For the parsers that use prebuilt lists of items, generate those
-        regexes now since all the parsers should be preloaded by the archive(s)
-        as well as being handed cmdline options and mapping file configuration.
-        """
-        for parser in self.parsers:
-            parser.generate_item_regexes()
-
-    def preload_all_archives_into_maps(self):
-        """Before doing the actual obfuscation, if we have multiple archives
-        to obfuscate then we need to preload each of them into the mappings
-        to ensure that node1 is obfuscated in node2 as well as node2 being
-        obfuscated in node1's archive.
-        """
-        self.log_info("Pre-loading all archives into obfuscation maps")
-        for _arc in self.report_paths:
-            for _parser in self.parsers:
-                try:
-                    pfile = _arc.prep_files[_parser.name.lower().split()[0]]
-                    if not pfile:
-                        continue
-                except (IndexError, KeyError):
-                    continue
-                if isinstance(pfile, str):
-                    pfile = [pfile]
-                for parse_file in pfile:
-                    self.log_debug("Attempting to load %s" % parse_file)
-                    try:
-                        content = _arc.get_file_content(parse_file)
-                        if not content:
-                            continue
-                        if isinstance(_parser, SoSUsernameParser):
-                            _parser.load_usernames_into_map(content)
-                        elif isinstance(_parser, SoSHostnameParser):
-                            if 'hostname' in parse_file:
-                                _parser.load_hostname_into_map(
-                                    content.splitlines()[0]
-                                )
-                            elif 'etc/hosts' in parse_file:
-                                _parser.load_hostname_from_etc_hosts(
-                                    content
-                                )
-                        else:
-                            for line in content.splitlines():
-                                self.obfuscate_line(line)
-                    except Exception as err:
-                        self.log_info(
-                            "Could not prepare %s from %s (archive: %s): %s"
-                            % (_parser.name, parse_file, _arc.archive_name,
-                               err)
-                        )
-
-    def obfuscate_report(self, archive):
+    def obfuscate_report(self, report):
         """Individually handle each archive or directory we've discovered by
         running through each file therein.
 
@@ -612,22 +482,25 @@ third party.
             :param report str:      Filepath to the directory or archive
         """
         try:
+            if not os.access(report, os.W_OK):
+                msg = "Insufficient permissions on %s" % report
+                self.log_info(msg)
+                self.ui_log.error(msg)
+                return
+
+            archive = SoSObfuscationArchive(report, self.tmpdir)
             arc_md = self.cleaner_md.add_section(archive.archive_name)
             start_time = datetime.now()
             arc_md.add_field('start_time', start_time)
-            # don't double extract nested archives
-            if not archive.is_extracted:
-                archive.extract()
+            archive.extract()
+            self.prep_maps_from_archive(archive)
             archive.report_msg("Beginning obfuscation...")
 
-            for fname in archive.get_file_list():
+            file_list = archive.get_file_list()
+            for fname in file_list:
                 short_name = fname.split(archive.archive_name + '/')[1]
                 if archive.should_skip_file(short_name):
                     continue
-                if (not self.opts.keep_binary_files and
-                        archive.should_remove_file(short_name)):
-                    archive.remove_file(short_name)
-                    continue
                 try:
                     count = self.obfuscate_file(fname, short_name,
                                                 archive.archive_name)
@@ -637,50 +510,66 @@ third party.
                     self.log_debug("Unable to parse file %s: %s"
                                    % (short_name, err))
 
-            try:
-                self.obfuscate_directory_names(archive)
-            except Exception as err:
-                self.log_info("Failed to obfuscate directories: %s" % err,
-                              caller=archive.archive_name)
-
-            try:
-                self.obfuscate_symlinks(archive)
-            except Exception as err:
-                self.log_info("Failed to obfuscate symlinks: %s" % err,
-                              caller=archive.archive_name)
-
             # if the archive was already a tarball, repack it
-            if not archive.is_nested:
-                method = archive.get_compression()
-                if method:
-                    archive.report_msg("Re-compressing...")
-                    try:
-                        archive.rename_top_dir(
-                            self.obfuscate_string(archive.archive_name)
-                        )
-                        archive.compress(method)
-                    except Exception as err:
-                        self.log_debug("Archive %s failed to compress: %s"
-                                       % (archive.archive_name, err))
-                        archive.report_msg("Failed to re-compress archive: %s"
-                                           % err)
-                        return
-                self.completed_reports.append(archive)
+            method = archive.get_compression()
+            if method:
+                archive.report_msg("Re-compressing...")
+                try:
+                    archive.rename_top_dir(
+                        self.obfuscate_string(archive.archive_name)
+                    )
+                    cmd = self.policy.get_cmd_for_compress_method(
+                        method,
+                        self.opts.threads
+                    )
+                    archive.compress(cmd)
+                except Exception as err:
+                    self.log_debug("Archive %s failed to compress: %s"
+                                   % (archive.archive_name, err))
+                    archive.report_msg("Failed to re-compress archive: %s"
+                                       % err)
+                    return
 
             end_time = datetime.now()
             arc_md.add_field('end_time', end_time)
             arc_md.add_field('run_time', end_time - start_time)
             arc_md.add_field('files_obfuscated', len(archive.file_sub_list))
             arc_md.add_field('total_substitutions', archive.total_sub_count)
-            rmsg = ''
-            if archive.removed_file_count:
-                rmsg = " [removed %s unprocessable files]"
-                rmsg = rmsg % archive.removed_file_count
-            archive.report_msg("Obfuscation completed%s" % rmsg)
+            self.completed_reports.append(archive)
+            archive.report_msg("Obfuscation completed")
 
         except Exception as err:
             self.ui_log.info("Exception while processing %s: %s"
-                             % (archive.archive_name, err))
+                             % (report, err))
+
+    def prep_maps_from_archive(self, archive):
+        """Open specific files from an archive and try to load those values
+        into our mappings before iterating through the entire archive.
+
+        Positional arguments:
+
+            :param archive SoSObfuscationArchive:   An open archive object
+        """
+        for parser in self.parsers:
+            if not parser.prep_map_file:
+                continue
+            prep_file = archive.get_file_path(parser.prep_map_file)
+            if not prep_file:
+                self.log_debug("Could not prepare %s: %s does not exist"
+                               % (parser.name, parser.prep_map_file),
+                               caller=archive.archive_name)
+                continue
+            # this is a bit clunky, but we need to load this particular
+            # parser in a different way due to how hostnames are validated for
+            # obfuscation
+            if isinstance(parser, SoSHostnameParser):
+                with open(prep_file, 'r') as host_file:
+                    hostname = host_file.readline().strip()
+                    parser.load_hostname_into_map(hostname)
+            if isinstance(parser, SoSUsernameParser):
+                parser.load_usernames_into_map(prep_file)
+            self.obfuscate_file(prep_file, parser.prep_map_file,
+                                archive.archive_name)
 
     def obfuscate_file(self, filename, short_name=None, arc_name=None):
         """Obfuscate and individual file, line by line.
@@ -699,124 +588,41 @@ third party.
         if not filename:
             # the requested file doesn't exist in the archive
             return
+        self.log_debug("Obfuscating %s" % short_name or filename,
+                       caller=arc_name)
         subs = 0
-        if not short_name:
-            short_name = filename.split('/')[-1]
-        if not os.path.islink(filename):
-            # don't run the obfuscation on the link, but on the actual file
-            # at some other point.
-            self.log_debug("Obfuscating %s" % short_name or filename,
-                           caller=arc_name)
-            tfile = tempfile.NamedTemporaryFile(mode='w', dir=self.tmpdir)
-            _parsers = [
-                _p for _p in self.parsers if not
-                any(
-                    _skip.match(short_name) for _skip in _p.skip_patterns
-                )
-            ]
-            with open(filename, 'r') as fname:
-                for line in fname:
-                    try:
-                        line, count = self.obfuscate_line(line, _parsers)
-                        subs += count
-                        tfile.write(line)
-                    except Exception as err:
-                        self.log_debug("Unable to obfuscate %s: %s"
-                                       % (short_name, err), caller=arc_name)
-            tfile.seek(0)
-            if subs:
-                shutil.copy(tfile.name, filename)
-            tfile.close()
-
-        _ob_short_name = self.obfuscate_string(short_name.split('/')[-1])
-        _ob_filename = short_name.replace(short_name.split('/')[-1],
-                                          _ob_short_name)
-
+        tfile = tempfile.NamedTemporaryFile(mode='w', dir=self.tmpdir)
+        with open(filename, 'r') as fname:
+            for line in fname:
+                if not line.strip():
+                    continue
+                try:
+                    line, count = self.obfuscate_line(line, short_name)
+                    subs += count
+                    tfile.write(line)
+                except Exception as err:
+                    self.log_debug("Unable to obfuscate %s: %s"
+                                   % (short_name, err), caller=arc_name)
+        tfile.seek(0)
+        if subs:
+            shutil.copy(tfile.name, filename)
+        tfile.close()
+        _ob_filename = self.obfuscate_string(short_name)
         if _ob_filename != short_name:
             arc_path = filename.split(short_name)[0]
             _ob_path = os.path.join(arc_path, _ob_filename)
-            # ensure that any plugin subdirs that contain obfuscated strings
-            # get created with obfuscated counterparts
-            if not os.path.islink(filename):
-                os.rename(filename, _ob_path)
-            else:
-                # generate the obfuscated name of the link target
-                _target_ob = self.obfuscate_string(os.readlink(filename))
-                # remove the unobfuscated original symlink first, in case the
-                # symlink name hasn't changed but the target has
-                os.remove(filename)
-                # create the newly obfuscated symlink, pointing to the
-                # obfuscated target name, which may not exist just yet, but
-                # when the actual file is obfuscated, will be created
-                os.symlink(_target_ob, _ob_path)
-
+            os.rename(filename, _ob_path)
         return subs
 
-    def obfuscate_symlinks(self, archive):
-        """Iterate over symlinks in the archive and obfuscate their names.
-        The content of the link target will have already been cleaned, and this
-        second pass over just the names of the links is to ensure we avoid a
-        possible race condition dependent on the order in which the link or the
-        target get obfuscated.
-
-        :param archive:     The archive being obfuscated
-        :type archive:      ``SoSObfuscationArchive``
-        """
-        self.log_info("Obfuscating symlink names", caller=archive.archive_name)
-        for symlink in archive.get_symlinks():
-            try:
-                # relative name of the symlink in the archive
-                _sym = symlink.split(archive.extracted_path)[1].lstrip('/')
-                self.log_debug("Obfuscating symlink %s" % _sym,
-                               caller=archive.archive_name)
-                # current target of symlink, again relative to the archive
-                _target = os.readlink(symlink)
-                # get the potentially renamed symlink name, this time the full
-                # path as it exists on disk
-                _ob_sym_name = os.path.join(archive.extracted_path,
-                                            self.obfuscate_string(_sym))
-                # get the potentially renamed relative target filename
-                _ob_target = self.obfuscate_string(_target)
-
-                # if either the symlink name or the target name has changed,
-                # recreate the symlink
-                if (_ob_sym_name != symlink) or (_ob_target != _target):
-                    os.remove(symlink)
-                    os.symlink(_ob_target, _ob_sym_name)
-            except Exception as err:
-                self.log_info("Error obfuscating symlink '%s': %s"
-                              % (symlink, err))
-
-    def obfuscate_directory_names(self, archive):
-        """For all directories that exist within the archive, obfuscate the
-        directory name if it contains sensitive strings found during execution
-        """
-        self.log_info("Obfuscating directory names in archive %s"
-                      % archive.archive_name)
-        for dirpath in sorted(archive.get_directory_list(), reverse=True):
-            for _name in os.listdir(dirpath):
-                _dirname = os.path.join(dirpath, _name)
-                _arc_dir = _dirname.split(archive.extracted_path)[-1]
-                if os.path.isdir(_dirname):
-                    _ob_dirname = self.obfuscate_string(_name)
-                    if _ob_dirname != _name:
-                        _ob_arc_dir = _arc_dir.rstrip(_name)
-                        _ob_arc_dir = os.path.join(
-                            archive.extracted_path,
-                            _ob_arc_dir.lstrip('/'),
-                            _ob_dirname
-                        )
-                        os.rename(_dirname, _ob_arc_dir)
-
     def obfuscate_string(self, string_data):
         for parser in self.parsers:
             try:
                 string_data = parser.parse_string_for_keys(string_data)
-            except Exception as err:
-                self.log_info("Error obfuscating string data: %s" % err)
+            except Exception:
+                pass
         return string_data
 
-    def obfuscate_line(self, line, parsers=None):
+    def obfuscate_line(self, line, filename):
         """Run a line through each of the obfuscation parsers, keeping a
         cumulative total of substitutions done on that particular line.
 
@@ -824,19 +630,16 @@ third party.
 
             :param line str:        The raw line as read from the file being
                                     processed
-            :param parsers:         A list of parser objects to obfuscate
-                                    with. If None, use all.
+            :param filename str:    Filename the line was read from
 
         Returns the fully obfuscated line and the number of substitutions made
         """
-        # don't iterate over blank lines, but still write them to the tempfile
-        # to maintain the same structure when we write a scrubbed file back
         count = 0
-        if not line.strip():
-            return line, count
-        if parsers is None:
-            parsers = self.parsers
-        for parser in parsers:
+        for parser in self.parsers:
+            if filename and any([
+                re.match(_s, filename) for _s in parser.skip_files
+            ]):
+                continue
             try:
                 line, _count = parser.parse_line(line)
                 count += _count
@@ -851,5 +654,3 @@ third party.
         for parser in self.parsers:
             _sec = parse_sec.add_section(parser.name.replace(' ', '_').lower())
             _sec.add_field('entries', len(parser.mapping.dataset.keys()))
-
-# vim: set et ts=4 sw=4 :
